// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ChargedFinalState.hh"
#include "Rivet/Projections/FastJets.hh"

namespace Rivet {

  // CMS UE charged particles vs. leading jet at 2.76 TeV
  class CMS_2015_I1385107 : public Analysis {
  public:
    /// Constructor
    CMS_2015_I1385107() : Analysis("CMS_2015_I1385107"),
                          etacut(2.0),
                          areatot((2 * etacut) * (2 * PI)),
                          area3(areatot / 3.),
                          area6(areatot / 6.) {
    }

  public:
    /// Book histograms and initialise projections before the run
    void init() {

      const ChargedFinalState cfs(-2.0, 2.0, 500*MeV);
      addProjection(cfs, "CFS");

      const ChargedFinalState cfsforjet(-2.5, 2.5, 500*MeV);
      const FastJets jetpro(cfsforjet, FastJets::SISCONE, 0.5);
      addProjection(jetpro, "Jets");

      _h_Nch_TransAVE_vs_pT = bookProfile1D(1, 1, 1); // Nch vs. pT_max      (TransAVE)
      _h_Sum_TransAVE_vs_pT = bookProfile1D(2, 1, 1); // sum(pT) vs. pT_max  (TransAVE)
      _h_Nch_TransMAX_vs_pT = bookProfile1D(3, 1, 1); // Nch vs. pT_max      (TransMAX)
      _h_Sum_TransMAX_vs_pT = bookProfile1D(4, 1, 1); // sum(pT) vs. pT_max  (TransMAX)
      _h_Nch_TransMIN_vs_pT = bookProfile1D(5, 1, 1); // Nch vs. pT_max      (TransMIN)
      _h_Sum_TransMIN_vs_pT = bookProfile1D(6, 1, 1); // sum(pT) vs. pT_max  (TransMIN)
      _h_Nch_TransDIF_vs_pT = bookProfile1D(7, 1, 1); // Nch vs. pT_max      (TransDIF)
      _h_Sum_TransDIF_vs_pT = bookProfile1D(8, 1, 1); // sum(pT) vs. pT_max  (TransDIF)
    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
      double weight = event.weight();

      // Find the lead jet, applying a restriction that the jets must be within |eta| < 2.
      FourMomentum p_lead;
      foreach (const Jet& j, applyProjection<FastJets>(event, "Jets").jetsByPt(1.0 * GeV)) {
        if (fabs(j.momentum().eta()) < 2.0) {
          p_lead = j.momentum();
          break;
        }
      }
      if (p_lead.isZero()) vetoEvent;
      double phi_lead = p_lead.phi();
      double pT_lead  = p_lead.pT();

      // Loop on charged particles and separate Left and Right transverse region
      ParticleVector particles = applyProjection<ChargedFinalState>(event, "CFS").particlesByPt();

      int nch_TransLeft = 0;
      int nch_TransRight = 0;
      double ptSum_TransLeft = 0.;
      double ptSum_TransRight = 0.;

      foreach (const Particle& p, particles) {
        double dphi = signedDeltaPhi(phi_lead, p.momentum().phi());
        if (dphi < -PI / 3. && dphi > -PI * 2. / 3.) {  // Transverse Right region
          nch_TransRight++;
          const double pT = p.momentum().pT() / GeV;
          ptSum_TransRight += pT;
        } else if (dphi > PI / 3. && dphi < PI * 2. / 3.) {  // Transverse Left region
          nch_TransLeft++;
          const double pT = p.momentum().pT() / GeV;
          ptSum_TransLeft += pT;
        }
      }

      // Translate to min and max (+sum and diff) Transverse region
      int nch_TransSUM = 0;
      int nch_TransDIF = 0;
      int nch_TransMIN = 0;
      int nch_TransMAX = 0;
      if (nch_TransRight > nch_TransLeft) {
        nch_TransMAX = nch_TransRight;
        nch_TransMIN = nch_TransLeft;
      } else {
        nch_TransMAX = nch_TransLeft;
        nch_TransMIN = nch_TransRight;
      }
      nch_TransSUM = nch_TransMAX + nch_TransMIN;
      nch_TransDIF = nch_TransMAX - nch_TransMIN;

      double ptSum_TransSUM = 0.;
      double ptSum_TransDIF = 0.;
      double ptSum_TransMIN = 0.;
      double ptSum_TransMAX = 0.;
      if (ptSum_TransRight > ptSum_TransLeft) {
        ptSum_TransMAX = ptSum_TransRight;
        ptSum_TransMIN = ptSum_TransLeft;
      } else {
        ptSum_TransMAX = ptSum_TransLeft;
        ptSum_TransMIN = ptSum_TransRight;
      }
      ptSum_TransSUM = ptSum_TransMAX + ptSum_TransMIN;
      ptSum_TransDIF = ptSum_TransMAX - ptSum_TransMIN;


      // Fill profiles
      _h_Nch_TransMIN_vs_pT->fill(pT_lead / GeV, 1. / area6 * nch_TransMIN, weight);
      _h_Sum_TransMIN_vs_pT->fill(pT_lead / GeV, 1. / area6 * ptSum_TransMIN, weight);

      _h_Nch_TransMAX_vs_pT->fill(pT_lead / GeV, 1. / area6 * nch_TransMAX, weight);
      _h_Sum_TransMAX_vs_pT->fill(pT_lead / GeV, 1. / area6 * ptSum_TransMAX, weight);

      _h_Nch_TransAVE_vs_pT->fill(pT_lead / GeV, 1. / area3 * nch_TransSUM, weight);
      _h_Sum_TransAVE_vs_pT->fill(pT_lead / GeV, 1. / area3 * ptSum_TransSUM, weight);

      _h_Nch_TransDIF_vs_pT->fill(pT_lead / GeV, 1. / area6 * nch_TransDIF, weight);
      _h_Sum_TransDIF_vs_pT->fill(pT_lead / GeV, 1. / area6 * ptSum_TransDIF, weight);
    }

    void finalize() {
    }


  private:
    // Data members like post-cuts event weight counters go here
    const double etacut;
    const double areatot;
    const double area3;
    const double area6;

    double signedDeltaPhi(double jetphi, double partphi) {
      double delta = partphi - jetphi;
      while (delta <= -PI) delta += 2 * PI;
      while (delta > PI) delta -= 2 * PI;
      return delta;
    }

  private:
    /// Histograms
    Profile1DPtr _h_Nch_TransAVE_vs_pT;
    Profile1DPtr _h_Sum_TransAVE_vs_pT;
    Profile1DPtr _h_Nch_TransDIF_vs_pT;
    Profile1DPtr _h_Sum_TransDIF_vs_pT;
    Profile1DPtr _h_Nch_TransMIN_vs_pT;
    Profile1DPtr _h_Sum_TransMIN_vs_pT;
    Profile1DPtr _h_Nch_TransMAX_vs_pT;
    Profile1DPtr _h_Sum_TransMAX_vs_pT;

  };


  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(CMS_2015_I1385107);

}
